var nbPalets = 0; //nombre de palets à afficher.
var largeurUnite; //unite de largeur en pixels pour les palets.
var hauteurPalet; //hauteur en pixel de chaque palet

var bar;
function resetBar() {
	bar = [{x: 120, palets: []},//abcisse des barres et palets qu'elles contiennent.
		{x: 300, palets:[]},
		{x: 480, palets: []}];
	for(var i = nbPalets; i>= 1; i--) {
		bar[0].palets.push(i);
	}
}


/**
 * 
 * @param xBarre abcisse de la barre
 * @param num numéro du palet, pour calculer sa largeur
 * @param pos position du palet sur la barre, entre 0 et nbPalets-1.
 * @param ctx le contexte de dessin.
 */
function dessinePalet(xBarre, num, pos, ctx) {
	//palet centré autour de la barre.
	ctx.fillRect(xBarre-(num*largeurUnite)/2,
			370-(pos+1)*hauteurPalet,
			num*largeurUnite,
			hauteurPalet);
	//contour
	ctx.strokeRect(xBarre-(num*largeurUnite)/2,
			370-(pos+1)*hauteurPalet,
			num*largeurUnite,
			hauteurPalet);
}

/**
 * Trace les barres ainsi que leurs palets.
 */
function dessineSituation() {
	var ctx = $('#canvas')[0].getContext('2d'); //objet pour dessiner.
	
	ctx.clearRect(0,0,600,369);//effacer les dessins précédents.
	
	ctx.beginPath();
	//redessine les barres :
	ctx.moveTo(bar[0].x, 370);
	ctx.lineTo(bar[0].x, 140);
	ctx.moveTo(bar[1].x, 370);
	ctx.lineTo(bar[1].x, 140);
	ctx.moveTo(bar[2].x, 370);
	ctx.lineTo(bar[2].x, 140);
	
	//pour chaque barre, on dessine les palets
	ctx.stroke();
	ctx.beginPath();
	ctx.fillStyle = "#bbb";
	for(var i = 0; i<=2; i++) {
		for(var j = 0; j < bar[i].palets.length; j++) {
			dessinePalet(bar[i].x, bar[i].palets[j], j, ctx);
		}
	}
	ctx.stroke();
}


var mouvements = []; //liste des mouvements à effectuer pour resoudre
var etat = 0; //position courante dans la lise des mouvements.


/**
 * indique la marche à suivre pour résoudre le problème
 * @param nb : nombre de palets à transférer
 * @param dep : barre de départ
 * @param arr : barre d'arrivée
 * @param int : barr intermédiaire
 */
function resoudre(nb,dep,arr,int) {
	if (nb > 0) {//si on a des palets à transférer...
		var a = resoudre(nb-1,dep,int,arr),//transfert de n-1 palets de départ vers intermédiaire en passant par arrivée.
			b = [[dep,arr]],//transfert du dernier palet de départ vers arrivée.
			c = resoudre(nb-1,int,arr,dep);//transfert de n-1 palets de intermédiaire vers arrivée en passant par départ.
		return a.concat(b).concat(c);
	}
	return [];
}

//préparation de l'animation.
function anime() {
	etat = 0;
	mouvements = resoudre(nbPalets,0,2,1);
}

//avancement graphique dans la liste des mouvements : on change le prochain mouvement en rouge.
function avanceMouvement() {
	currentLine = $('.current-mvt');
	currentLine.removeClass('current-mvt');
	currentLine = currentLine.next('li').addClass('current-mvt');
	if (currentLine.length && currentLine.position().top > 200) {
		$('.mvt-wrapper')[0].scrollTop += 100;
	}
}

//on avance dans la liste des mouvements.
function avancer() {
	var mvt;
	if (etat === -1) {//on vient de reset.
		dessineSituation();
		etat++;
		//avanceMouvement()
	}
	else if (etat < mouvements.length) {
		mvt = mouvements[etat];
		bar[mvt[1]].palets.push(bar[mvt[0]].palets.pop());
		dessineSituation();
		etat++;
		avanceMouvement()
	} 
	if (etat == mouvements.length) {//fin des mouvements
		if (!$("#boucle").prop('checked')) {//si la lecture n'est pas en boucle.
			$("#move, #launch, #pause").addClass('activeSort');
			if (anim) {
				window.clearInterval(anim);//arret de l'éventuelle animation.
			}
		} else {
			etat = -1;//reset l'animation.
			resetBar();
			window.clearInterval(anim);
			$("#launch").removeClass('activeSort').click();
		}
	}
}

/**
 * Image avant animation
 */
function dessineBase() {
	var ctx = $('#canvas')[0].getContext('2d'); //objet pour dessiner.
	
	ctx.clearRect(0,0,600,400);//effacer les dessins précédents.
	ctx.linewidth = 10;
	ctx.strokeStyle = "#000";
	//dessin de la base
	ctx.moveTo(30,370);
	ctx.lineTo(570,370);
	//et des barres.
	ctx.moveTo(bar[0].x,370);
	ctx.lineTo(bar[0].x,140);
	ctx.moveTo(bar[1].x,370);
	ctx.lineTo(bar[1].x,140);
	ctx.moveTo(bar[2].x,370);
	ctx.lineTo(bar[2].x,140);
	ctx.stroke();
}

function listeMvt() {
	var wrap = $("#show-mvt .mvt-wrapper"),
		$ul = $('<ol>'),
		n = mouvements.length;
	wrap.empty();
	for(var i = 0; i <  n; i++) {
		$ul.append('<li>' + mouvements[i][0] + ' -> ' + mouvements[i][1] + '</li>');
	}
	wrap.append($ul);
}

var anim;
var interval = 400;
$(document).ready(function () {
	resetBar();
	dessineBase();
	$("#draw").click(function () {
		$("#pause").click();
		nbPalets = parseInt($("#num").val());//au click sur le bouton d'animation, on change la valeur de palets.
		if (nbPalets != 0) {
			largeurUnite = Math.floor(160 / nbPalets); //calcul de la largeur optimale des palets
			hauteurPalet = Math.min(30,Math.floor(210/nbPalets)); //idem avec la hauteur.
			resetBar();
			dessineSituation();
			anime(); //puis on prépare l'animation.
			$("#move, #launch").removeClass('activeSort');
			interval = parseInt($("#timeInterval").val());
			listeMvt();
			$('.mvt-wrapper li:first').addClass('current-mvt');
			$('.mvt-wrapper')[0].scrollTop = 0;
		}
	});
	$('#move').click(avancer);
	$('#launch').click(function () {
		if (!$(this).hasClass('activeSort')) {
			//var currentLine = $('.mvt-wrapper li:first').addClass('current-mvt');
			
			anim = setInterval(function(){
					
					avancer();
					
				}
				,interval);
			$("#pause").removeClass("activeSort");
			$(this).addClass('activeSort');
		}
	});
	$("#pause").click(function () {
		if (anim) {
			window.clearInterval(anim);
		}
		$("#launch").removeClass("activeSort");
		$(this).addClass('activeSort');
	});
	$('#show-btn').click(function() {
		$(this).toggleClass('activeSort');
		$(this).parent().find('.mvt-wrapper').toggle('fast');
	});
});